"""Unit tests for ``accretion_common.venv_magic.uploader``."""
from typing import Dict, Iterable

import pytest

from accretion_common.venv_magic import uploader

pytestmark = [pytest.mark.local, pytest.mark.functional]


@pytest.mark.parametrize(
    "installed, runtime_name, expected_hash",
    (
            (
                    (
                        dict(Name="asdf", Details="123"),
                        dict(Name="zz", Details="333"),
                        dict(Name="ff", Details="[3214];wutwut")
                    ),
                    "python",
                    "2babf317cda24b98133b8937ca952e64154f3c00a14bb83b7c20a2e2b13a0237",
            ),
    )
)
def test_key_hash(installed: Iterable[Dict[str, str]], runtime_name: str, expected_hash: str):
    test = uploader._key_hash(installed=installed, runtime_name=runtime_name, force_new=False)
    test_unique = uploader._key_hash(installed=installed, runtime_name=runtime_name, force_new=True)

    assert test == expected_hash
    assert test_unique != expected_hash



"""Common internal Accretion utilities."""
from typing import Dict

import attr
from pkg_resources import Requirement

__all__ = ("PackageDetails",)


@attr.s(auto_attribs=True)
class PackageDetails:
    """Container for information identifying a package.

    :param str name: Package name
    :param str details: Package version or other identifying details
    """

    Name: str
    Details: str = attr.Factory(str)

    @classmethod
    def from_pip_log(cls, log_value: str) -> "PackageDetails":
        """Read package name and version from a ``pip install`` log."""
        name, version = log_value.rsplit("-", 1)
        return PackageDetails(name, version)

    def to_dict(self) -> Dict[str, str]:
        """Pack information into a dictionary."""
        return dict(Name=self.Name, Details=self.Details)

    @classmethod
    def from_requirements_entry(cls, requirement_line: str) -> "PackageDetails":
        """Read package name and version from a ``requirements.txt`` file entry."""
        entry = Requirement.parse(requirement_line)
        name = entry.project_name

        details = requirement_line[len(name) :]
        return PackageDetails(Name=name, Details=details)
